import datetime
import glob
import json
import logging
import os
import socket
import subprocess

import requests
from django.conf import settings
from django.contrib.auth.forms import AuthenticationForm
from django.core.files.storage import default_storage
from django.db.models import Q
from django.http import HttpResponseForbidden, JsonResponse
from django.views.generic.base import TemplateView
from emissions.models import Nonstop
from nonstop.models import SomaLogLine, Track

from panikdb.context_processors import internal_ip
from panikdb.pige.views import PigeDownloadMixin
from panikdb.service_messages.models import Message

logger = logging.getLogger('panikdb')


def is_regie_computer(request):
    return internal_ip(request).get('current_ip') in settings.REGIE_IPS


class RegieHome(TemplateView, PigeDownloadMixin):
    template_name = 'regie-home.html'

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context.update(self.get_pige_context_data())
        context['login_form'] = AuthenticationForm()

        context['service_message'] = Message.objects.all().first()

        if (
            getattr(settings, 'NONSTOP_ON_AIR_SWITCH_URL', None)
            or getattr(settings, 'NONSTOP_JACK_SWITCH_SEND_UDP_NOTIFICATIONS', None)
        ) and getattr(self, 'request', None):
            context['is_regie_computer'] = is_regie_computer(self.request)
            context['has_switch'] = True

        return context


regie_home = RegieHome.as_view()


def json_tracks(tracks):
    return JsonResponse(
        {
            'data': [
                {
                    'id': track.id,
                    'title': track.title,
                    'artist': track.artist.name if track.artist_id else None,
                    'instru': track.instru,
                    'language': track.language,
                    'duration': '%d:%02d'
                    % (track.duration.total_seconds() / 60, track.duration.total_seconds() % 60),
                }
                for track in tracks
            ]
        }
    )


def regie_tracks(request):
    now = datetime.datetime.now()
    current_zone = None
    for nonstop in Nonstop.objects.all():
        if (nonstop.start < nonstop.end and (now.time() >= nonstop.start and now.time() < nonstop.end)) or (
            nonstop.start > nonstop.end and (now.time() >= nonstop.start or now.time() < nonstop.end)
        ):
            current_zone = nonstop
            break

    tracks = Track.objects.filter(duration__isnull=False)
    tracks = tracks.filter(duration__gte=datetime.timedelta(minutes=2, seconds=30))
    tracks = tracks.filter(duration__lt=datetime.timedelta(minutes=4, seconds=30))
    if current_zone:
        tracks = tracks.filter(nonstop_zones__in=[current_zone.id])
    tracks = tracks.exclude(language__isnull=True).exclude(language='')
    tracks = tracks.order_by('?')
    return json_tracks(tracks[:5])


def regie_tracks_search(request):
    q = request.GET['q']
    if q.startswith('https://'):
        ytdl_path = default_storage.path('ytdl')
        if not os.path.exists(ytdl_path):
            os.mkdir(ytdl_path)
        # make a first run with just metadata, this is to obtain the filename
        # and use a file that was previously downloaded.
        proc = subprocess.run(
            [
                'youtube-dl',
                '--extract-audio',
                '--no-playlist',
                '--no-mtime',
                '--dump-json',
                '--audio-format',
                'opus',
                '-o',
                ytdl_path + '/%(id)s.%(ext)s',
                q,
            ],
            capture_output=True,
        )
        output = json.loads(proc.stdout)
        filename = output['_filename']
        # audio conversion may be done by youtube-dl and filename would still
        # contain the original filename, look for files with same basename,
        # regardless of the extension.
        base_filename = os.path.splitext(filename)[0]
        try:
            real_filename = [
                x
                for x in glob.glob(base_filename + '*')
                if not x.endswith('.part') and not x.endswith('.json')
            ][0]
        except IndexError:
            # not yet downloaded, store json for async downloading with the ytdl
            # management command
            ready = False
            real_filename = os.path.splitext(filename)[0] + '.json'
            output['_url'] = q
            with open(real_filename, 'w') as fd:
                json.dump(output, fp=fd, indent=2)
        else:
            ready = True
        url = os.path.join(settings.MEDIA_URL, 'ytdl', real_filename[len(ytdl_path) + 1 :])
        title = output['title']
        duration = '%d:%02d' % (int(output['duration']) / 60, int(output['duration']) % 60)
        good_filename = '%s%s' % (title, os.path.splitext(real_filename)[-1])
        return JsonResponse(
            {
                'data': [
                    {
                        'title': title,
                        'duration': duration,
                        'url': url,
                        'filename': good_filename,
                        'ready': ready,
                    }
                ]
            }
        )

    tracks = Track.objects.filter(duration__isnull=False).filter(
        Q(title__icontains=q.lower()) | Q(artist__name__icontains=q.lower())
    )
    tracks = tracks.order_by('?')
    return json_tracks(tracks[:5])


def playing(request):
    try:
        latest_played = SomaLogLine.objects.select_related('track', 'track__artist').latest('play_timestamp')
    except SomaLogLine.DoesNotExist:
        return JsonResponse({'err': 1})
    duration = latest_played.track.duration
    elapsed = datetime.datetime.now() - latest_played.play_timestamp
    remaining = duration - elapsed
    if remaining.total_seconds() < 2:
        return JsonResponse({})
    elif remaining.total_seconds() < 0:
        remaining = datetime.timedelta(seconds=0)
    response = {}
    if latest_played.track.artist:
        response['artist'] = latest_played.track.artist.name
    response['title'] = latest_played.track.title
    response['duration'] = duration.seconds
    response['elapsed'] = int(elapsed.total_seconds())
    return JsonResponse(response)


def switch(request, *args, **kwargs):
    if not is_regie_computer(request):
        return HttpResponseForbidden()
    logger.info('regie switch to %s', request.GET['selection'])
    if getattr(settings, 'NONSTOP_ON_AIR_SWITCH_URL', None):
        requests.post(settings.NONSTOP_ON_AIR_SWITCH_URL, data={'s': request.GET['selection']})
    if getattr(settings, 'NONSTOP_JACK_SWITCH_SEND_UDP_NOTIFICATIONS', False):
        sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)  # UDP
        sock.sendto(
            json.dumps({'active': int(request.GET['selection'])}).encode(),
            settings.NONSTOP_JACK_SWITCH_SEND_UDP_NOTIFICATIONS,
        )
    return JsonResponse({'err': 0})
